@@ -36,9 +36,10 @@ from khal.khalendar.event import Event
 from khal.khalendar.exceptions import DuplicateUid, ReadOnlyCalendarError
 
 from .exceptions import ConfigurationError
+from .icalendar import (cal_from_ics, new_event as new_vevent, split_ics,
+                        sort_key as sort_vevent_key)
 from .khalendar.vdir import Item
 from .terminal import merge_columns
-from .utils import cal_from_ics
 
 logger = logging.getLogger('khal')
 
@@ -187,9 +188,9 @@ def get_events_between(
         # to understand what's going on here this way
         if notstarted:
             if event.allday and event.start < original_start.date():
-                    continue
+                continue
             elif not event.allday and event.start_local < original_start:
-                    continue
+                continue
         if seen is not None and event.uid in seen:
             continue
 
@@ -280,7 +281,10 @@ def new_interactive(collection, calendar_name, conf, info, location=None,
                     format=None, env=None):
     try:
         info = parse_datetime.eventinfofstr(
-            info, conf['locale'], adjust_reasonably=True, localize=False,
+            info, conf['locale'],
+            conf['default']['default_event_duration'],
+            conf['default']['default_dayevent_duration'],
+            adjust_reasonably=True, localize=False,
         )
     except DateTimeParseError:
         info = dict()
@@ -341,7 +345,10 @@ def new_from_string(collection, calendar_name, conf, info, location=None,
                     format=None, env=None):
     """construct a new event from a string and add it"""
     info = parse_datetime.eventinfofstr(
-        info, conf['locale'], adjust_reasonably=True, localize=False,
+        info, conf['locale'],
+        conf['default']['default_event_duration'],
+        conf['default']['default_dayevent_duration'],
+        adjust_reasonably=True, localize=False
     )
     new_from_args(
         collection, calendar_name, conf, format=format, env=env,
@@ -355,8 +362,10 @@ def new_from_args(collection, calendar_name, conf, dtstart=None, dtend=None,
                   categories=None, repeat=None, until=None, alarms=None,
                   timezone=None, format=None, env=None):
     """Create a new event from arguments and add to vdirs"""
+    if isinstance(categories, str):
+        categories = list([category.strip() for category in categories.split(',')])
     try:
-        event = utils.new_event(
+        event = new_vevent(
             locale=conf['locale'], location=location, categories=categories,
             repeat=repeat, until=until, alarms=alarms, dtstart=dtstart,
             dtend=dtend, summary=summary, description=description, timezone=timezone,
@@ -495,7 +504,10 @@ def edit_event(event, collection, locale, allow_quit=False, width=80):
             value = prompt(question, default)
             if allow_none and value == "None":
                 value = ""
-            getattr(event, "update_" + attr)(value)
+            if attr == 'categories':
+                getattr(event, "update_" + attr)(list([cat.strip() for cat in value.split(',')]))
+            else:
+                getattr(event, "update_" + attr)(value)
             edited = True
 
         if edited:
@@ -515,9 +527,9 @@ def edit(collection, search_string, locale, format=None, allow_past=False, conf=
     for event in events:
         if not allow_past:
             if event.allday and event.end < now.date():
-                    continue
+                continue
             elif not event.allday and event.end_local < now:
-                    continue
+                continue
         event_text = textwrap.wrap(event.format(format, relative_to=now), term_width)
         echo(''.join(event_text))
         if not edit_event(event, collection, locale, allow_quit=True, width=term_width):
@@ -549,7 +561,7 @@ def import_ics(collection, conf, ics, batch=False, random_uid=False, format=None
     """
     if format is None:
         format = conf['view']['event_format']
-    vevents = utils.split_ics(ics, random_uid, conf['locale']['default_timezone'])
+    vevents = split_ics(ics, random_uid, conf['locale']['default_timezone'])
     for vevent in vevents:
         import_event(vevent, collection, conf['locale'], batch, format, env)
 
@@ -619,7 +631,7 @@ def print_ics(conf, name, ics, format):
 
     vevents = list()
     for uid in events_grouped:
-        vevents.append(sorted(events_grouped[uid], key=utils.sort_key))
+        vevents.append(sorted(events_grouped[uid], key=sort_vevent_key))
 
     echo('{} events found in {}'.format(len(vevents), name))
     for sub_event in vevents:
