@@ -20,32 +20,28 @@
 # WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 """
 The SQLite backend implementation.
-
-note on naming:
-  * every variable name vevent should be of type icalendar.Event
-  * every variable named event should be of type khal.khalendar.Events
-  * variables named vevents/events (plural) should be iterables of their
-    respective types
-
 """
-# TODO remove creating Events from SQLiteDb
-# we currently expect str/CALENDAR objects but return Event(), we should
-# accept and return the same kind of events
-import contextlib
-from datetime import datetime, timedelta
-from os import makedirs, path
-import sqlite3
 
-from dateutil import parser
+import contextlib
+import datetime as dt
+from enum import IntEnum
+import logging
+import sqlite3
+from os import makedirs, path
+from typing import Any, Dict, Iterable, List, Optional, Tuple
+
 import icalendar
 import pytz
+from dateutil import parser
 
-from .event import Event, EventStandIn
-from . import utils
-from .. import log
-from .exceptions import CouldNotCreateDbDir, OutdatedDbVersionError, UpdateFailed
+from .. import utils
+from ..icalendar import (cal_from_ics, assert_only_one_uid,
+                         expand as expand_vevent, sanitize as sanitize_vevent,
+                         sort_key as sort_vevent_key)
+from .exceptions import (CouldNotCreateDbDir, OutdatedDbVersionError,
+                         UpdateFailed, NonUniqueUID)
 
-logger = log.logger
+logger = logging.getLogger('khal')
 
 DB_VERSION = 5  # The current db layout version
 
@@ -53,51 +49,34 @@ RECURRENCE_ID = 'RECURRENCE-ID'
 THISANDFUTURE = 'THISANDFUTURE'
 THISANDPRIOR = 'THISANDPRIOR'
 
-DATE = 0
-DATETIME = 1
-
 PROTO = 'PROTO'
 
 
-def sort_key(vevent):
-    """helper function to determine order of VEVENTS
-
-    so that recurrence-id events come after the corresponding rrule event, etc
-
-    :param vevent: icalendar.Event
-    :rtype: tuple(str, int)
-    """
-    assert isinstance(vevent, icalendar.Event)
-    uid = str(vevent['UID'])
-    rec_id = vevent.get(RECURRENCE_ID)
-    if rec_id is None:
-        return uid, 0
-    rrange = rec_id.params.get('RANGE')
-    if rrange == THISANDFUTURE:
-        return uid, utils.to_unix_time(rec_id.dt)
-    else:
-        return uid, 1
+class EventType(IntEnum):
+    DATE = 0
+    DATETIME = 1
 
 
 class SQLiteDb(object):
     """
     This class should provide a caching database for a calendar, keeping raw
-    vevents in one table but allowing to retrieve events by dates (via the help
+    vevents in one table and allowing to retrieve them by dates (via the help
     of some auxiliary tables)
 
-    :param calendar: the `name` of this calendar, if the same *name* and
-                     *dbpath* is given on next creation of an SQLiteDb object
-                     the same tables will be used
-    :type calendar: str
+    :param calendar: names of calendars to select from, those are used as
+        additional itentifiers together with event's uids. Each (uid, calendar)
+        combination should be unique.
     :param db_path: path where this sqlite database will be saved, if this is
-                    None, a place according to the XDG specifications will be
-                    chosen
-    :type db_path: str or None
+        None, a place according to the XDG specifications will be chosen
     """
 
-    def __init__(self, calendars, db_path, locale):
+    def __init__(self,
+                 calendars: Iterable[str],
+                 db_path: Optional[str],
+                 locale: Dict[str, str],
+                 ) -> None:
         assert db_path is not None
-        self.calendars = calendars
+        self.calendars = list(calendars)
         self.db_path = path.expanduser(db_path)
         self._create_dbdir()
         self.locale = locale
@@ -108,24 +87,20 @@ class SQLiteDb(object):
         self._check_calendars_exists()
         self._check_table_version()
 
-    @property
-    def _select_calendars(self):
-        return ', '.join(['\'' + cal + '\'' for cal in self.calendars])
-
     @contextlib.contextmanager
     def at_once(self):
         assert not self._at_once
         self._at_once = True
         try:
             yield self
-        except:
+        except:  # noqa
             raise
         else:
             self.conn.commit()
         finally:
             self._at_once = False
 
-    def _create_dbdir(self):
+    def _create_dbdir(self) -> None:
         """create the dbdir if it doesn't exist"""
         if self.db_path == ':memory:':
             return None
@@ -136,10 +111,10 @@ class SQLiteDb(object):
                 makedirs(dbdir, mode=0o770)
                 logger.debug('success')
             except OSError as error:
-                logger.fatal('failed to create {0}: {1}'.format(dbdir, error))
+                logger.critical('failed to create {0}: {1}'.format(dbdir, error))
                 raise CouldNotCreateDbDir()
 
-    def _check_table_version(self):
+    def _check_table_version(self) -> None:
         """tests for current db Version
         if the table is still empty, insert db_version
         """
@@ -155,7 +130,7 @@ class SQLiteDb(object):
                 " is probably an invalid or outdated database.\n"
                 "You should consider removing it and running khal again.")
 
-    def _create_default_tables(self):
+    def _create_default_tables(self) -> None:
         """creates version and calendar tables and inserts table version number
         """
         self.cursor.execute('CREATE TABLE IF NOT EXISTS '
@@ -197,13 +172,12 @@ class SQLiteDb(object):
             );''')
         self.conn.commit()
 
-    def _check_calendars_exists(self):
+    def _check_calendars_exists(self) -> None:
         """make sure an entry for the current calendar exists in `calendar`
         table
         """
         for cal in self.calendars:
-            self.cursor.execute('''SELECT count(*) FROM calendars
-                    WHERE calendar = ?;''', (cal,))
+            self.cursor.execute('''SELECT count(*) FROM calendars WHERE calendar = ?;''', (cal,))
             result = self.cursor.fetchone()
 
             if result[0] != 0:
@@ -213,7 +187,7 @@ class SQLiteDb(object):
                 stuple = (cal, '')
                 self.sql_ex(sql_s, stuple)
 
-    def sql_ex(self, statement, stuple=''):
+    def sql_ex(self, statement: str, stuple: tuple) -> List:
         """wrapper for sql statements, does a "fetchall" """
         self.cursor.execute(statement, stuple)
         result = self.cursor.fetchall()
@@ -221,34 +195,38 @@ class SQLiteDb(object):
             self.conn.commit()
         return result
 
-    def update(self, vevent_str, href, etag='', calendar=None):
-        """insert a new or update an existing card in the db
+    def update(self, vevent_str: str, href: str, etag: str='', calendar: str=None) -> None:
+        """insert a new or update an existing event into the db
 
         This is mostly a wrapper around two SQL statements, doing some cleanup
         before.
 
         :param vevent_str: event to be inserted or updated.
-                           We assume that even if it contains more than one
-                           VEVENT, that they are all part of the same event and
-                           all have the same UID
-        :type vevent: unicode
+            We assume that even if it contains more than one VEVENT, that they
+            are all part of the same event and all have the same UID
         :param href: href of the card on the server, if this href already
-                     exists in the db the card gets updated. If no href is
-                     given, a random href is chosen and it is implied that this
-                     card does not yet exist on the server, but will be
-                     uploaded there on next sync.
-        :type href: str()
+            exists in the db the card gets updated. If no href is given, a
+            random href is chosen and it is implied that this card does not yet
+            exist on the server, but will be uploaded there on next sync.
         :param etag: the etag of the vcard, if this etag does not match the
-                     remote etag on next sync, this card will be updated from
-                     the server. For locally created vcards this should not be
-                     set
-        :type etag: str()
+            remote etag on next sync, this card will be updated from the server.
+            For locally created vcards this should not be set
         """
         assert calendar is not None
         assert href is not None
-        ical = icalendar.Event.from_ical(vevent_str)
+        ical = cal_from_ics(vevent_str)
         check_for_errors(ical, calendar, href)
-        vevents = (utils.sanitize(c, self.locale['default_timezone'], href, calendar) for
+        if not assert_only_one_uid(ical):
+            logger.warning(
+                "The .ics file at {}/{} contains multiple UIDs.\n"
+                "This should not occur in vdir .ics files.\n"
+                "If you didn't edit the file by hand, please report a bug "
+                "at https://github.com/pimutils/khal/issues .\n"
+                "If you want to import it, please use `khal import FILE`."
+                "".format(calendar, href)
+            )
+            raise NonUniqueUID
+        vevents = (sanitize_vevent(c, self.locale['default_timezone'], href, calendar) for
                    c in ical.walk() if c.name == 'VEVENT')
         # Need to delete the whole event in case we are updating a
         # recurring event with an event which is either not recurring any
@@ -256,71 +234,94 @@ class SQLiteDb(object):
         # tables. There are obviously better ways to achieve the same
         # result.
         self.delete(href, calendar=calendar)
-        for vevent in sorted(vevents, key=sort_key):
+        for vevent in sorted(vevents, key=sort_vevent_key):
             check_for_errors(vevent, calendar, href)
             check_support(vevent, href, calendar)
             self._update_impl(vevent, href, calendar)
 
-        sql_s = ('INSERT INTO events '
-                 '(item, etag, href, calendar) '
-                 'VALUES (?, ?, ?, ?);')
+        sql_s = ('INSERT INTO events (item, etag, href, calendar) VALUES (?, ?, ?, ?);')
         stuple = (vevent_str, etag, href, calendar)
         self.sql_ex(sql_s, stuple)
 
-    def update_birthday(self, vevent, href, etag='', calendar=None):
-        """
-        XXX write docstring
+    def update_vcf_dates(self, vevent_str: str, href: str, etag: str='',
+                         calendar: str=None) -> None:
+        """insert events from a vcard into the db
+
+        This is will parse BDAY, ANNIVERSARY, X-ANNIVERSARY and X-ABDATE fields.
+        It will also look for any X-ABLABEL fields associated with an X-ABDATE
+        and use that in the event description.
+
+        :param vevent_str: contact (vcard) to be parsed.
+        :param href: href of the card on the server, if this href already
+            exists in the db the card gets updated. If no href is given, a
+            random href is chosen and it is implied that this card does not yet
+            exist on the server, but will be uploaded there on next sync.
+        :param etag: the etag of the vcard, if this etag does not match the
+            remote etag on next sync, this card will be updated from the server.
+            For locally created vcards this should not be set
         """
         assert calendar is not None
         assert href is not None
-        self.delete(href, calendar=calendar)
-        ical = icalendar.Event.from_ical(vevent)
+        # Delete all event entries for this contact
+        self.deletelike(href + '%', calendar=calendar)
+        ical = cal_from_ics(vevent_str)
         vcard = ical.walk()[0]
-        if 'BDAY' in vcard.keys():
-            bday = vcard['BDAY']
-            if isinstance(bday, list):
-                logger.warning(
-                    'Vcard {0} in collection {1} has more than one '
-                    'BIRTHDAY, will be skipped and not be available '
-                    'in khal.'.format(href, calendar)
-                )
-                return
-            try:
-                if bday[0:2] == '--' and bday[3] != '-':
-                    bday = '1900' + bday[2:]
-                    orig_bday = False
+        for key in vcard.keys():
+            if key in ['BDAY', 'X-ANNIVERSARY', 'ANNIVERSARY'] or key.endswith('X-ABDATE'):
+                date = vcard[key]
+                if isinstance(date, list):
+                    logger.warning(
+                        'Vcard {0} in collection {1} has more than one '
+                        '{2}, will be skipped and not be available '
+                        'in khal.'.format(href, calendar, key)
+                    )
+                    continue
+                try:
+                    if date[0:2] == '--' and date[3] != '-':
+                        date = '1900' + date[2:]
+                        orig_date = False
+                    else:
+                        orig_date = True
+                    date = parser.parse(date).date()
+                except ValueError:
+                    logger.warning(
+                        'cannot parse {0} in {1} in collection {2}'.format(key, href, calendar))
+                    continue
+                if 'FN' in vcard:
+                    name = vcard['FN']
                 else:
-                    orig_bday = True
-                bday = parser.parse(bday).date()
-            except ValueError:
-                logger.warning(
-                    'cannot parse BIRTHDAY in {0} in collection {1}'.format(href, calendar))
-                return
-            if 'FN' in vcard:
-                name = vcard['FN']
-            else:
-                n = vcard['N'].split(';')
-                name = ' '.join([n[1], n[2], n[0]])
-            event = icalendar.Event()
-            event.add('dtstart', bday)
-            event.add('dtend', bday + timedelta(days=1))
-            if bday.month == 2 and bday.day == 29:  # leap year
-                event.add('rrule', {'freq': 'YEARLY', 'BYYEARDAY': 60})
-            else:
-                event.add('rrule', {'freq': 'YEARLY'})
-            if orig_bday:
-                event.add('x-birthday',
-                          '{:04}{:02}{:02}'.format(bday.year, bday.month, bday.day))
-                event.add('x-fname', name)
-            event.add('summary', '{0}\'s birthday'.format(name))
-            event.add('uid', href)
-            event_str = event.to_ical().decode('utf-8')
-            self._update_impl(event, href, calendar)
-            sql_s = ('INSERT INTO events (item, etag, href, calendar) VALUES (?, ?, ?, ?);')
-            stuple = (event_str, etag, href, calendar)
-            self.sql_ex(sql_s, stuple)
+                    n = vcard['N'].split(';')
+                    name = ' '.join([n[1], n[2], n[0]])
+                vevent = icalendar.Event()
+                vevent.add('dtstart', date)
+                vevent.add('dtend', date + dt.timedelta(days=1))
+                if date.month == 2 and date.day == 29:  # leap year
+                    vevent.add('rrule', {'freq': 'YEARLY', 'BYYEARDAY': 60})
+                else:
+                    vevent.add('rrule', {'freq': 'YEARLY'})
+                description = get_vcard_event_description(vcard, key)
+                if orig_date:
+                    if key == 'BDAY':
+                        xtag = 'x-birthday'
+                    elif key.endswith('ANNIVERSARY'):
+                        xtag = 'x-anniversary'
+                    else:
+                        xtag = 'x-abdate'
+                        vevent.add('x-ablabel', description)
+                    vevent.add(xtag,
+                               '{:04}{:02}{:02}'.format(date.year, date.month, date.day))
+                    vevent.add('x-fname', name)
+                vevent.add('summary',
+                           '{0}\'s {1}'.format(name, description))
+                vevent.add('uid', href + key)
+                vevent_str = vevent.to_ical().decode('utf-8')
+                self._update_impl(vevent, href + key, calendar)
+                sql_s = ('INSERT INTO events (item, etag, href, calendar)'
+                         ' VALUES (?, ?, ?, ?);')
+                stuple = (vevent_str, etag, href + key, calendar)
+                self.sql_ex(sql_s, stuple)
 
-    def _update_impl(self, vevent, href, calendar):
+    def _update_impl(self, vevent: icalendar.cal.Event, href: str, calendar: str) -> None:
         """insert `vevent` into the database
 
         expand `vevent`'s recurrence rules (if needed) and insert all instance
@@ -336,11 +337,11 @@ class SQLiteDb(object):
             rrange = rec_id.params.get('RANGE')
 
         # testing on datetime.date won't work as datetime is a child of date
-        if not isinstance(vevent['DTSTART'].dt, datetime):
-            dtype = DATE
+        if not isinstance(vevent['DTSTART'].dt, dt.datetime):
+            dtype = EventType.DATE
         else:
-            dtype = DATETIME
-        if ('TZID' in vevent['DTSTART'].params and dtype == DATETIME) or \
+            dtype = EventType.DATETIME
+        if ('TZID' in vevent['DTSTART'].params and dtype == EventType.DATETIME) or \
                 getattr(vevent['DTSTART'].dt, 'tzinfo', None):
             recs_table = 'recs_loc'
         else:
@@ -349,10 +350,10 @@ class SQLiteDb(object):
         thisandfuture = (rrange == THISANDFUTURE)
         if thisandfuture:
             start_shift, duration = calc_shift_deltas(vevent)
-            start_shift = start_shift.days * 3600 * 24 + start_shift.seconds
-            duration = duration.days * 3600 * 24 + duration.seconds
+            start_shift_seconds = start_shift.days * 3600 * 24 + start_shift.seconds
+            duration_seconds = duration.days * 3600 * 24 + duration.seconds
 
-        dtstartend = utils.expand(vevent, href)
+        dtstartend = expand_vevent(vevent, href)
         if not dtstartend:
             # Does this event even have dates? Technically it is possible for
             # events to be empty/non-existent by deleting all their recurrences
@@ -360,7 +361,7 @@ class SQLiteDb(object):
             return
 
         for dtstart, dtend in dtstartend:
-            if dtype == DATE:
+            if dtype == EventType.DATE:
                 dbstart = utils.to_unix_time(dtstart)
                 dbend = utils.to_unix_time(dtend)
             else:
@@ -370,23 +371,27 @@ class SQLiteDb(object):
             if rec_id is not None:
                 ref = rec_inst = str(utils.to_unix_time(rec_id.dt))
             else:
-                rec_inst = dbstart
+                rec_inst = str(dbstart)
                 ref = PROTO
 
             if thisandfuture:
                 recs_sql_s = (
                     'UPDATE {0} SET dtstart = rec_inst + ?, dtend = rec_inst + ?, ref = ? '
                     'WHERE rec_inst >= ? AND href = ? AND calendar = ?;'.format(recs_table))
-                stuple = (start_shift, start_shift + duration, ref, rec_inst, href, calendar)
+                stuple_f = (
+                    start_shift_seconds, start_shift_seconds + duration_seconds,
+                    ref, rec_inst, href, calendar,
+                )
+                self.sql_ex(recs_sql_s, stuple_f)
             else:
                 recs_sql_s = (
                     'INSERT OR REPLACE INTO {0} '
                     '(dtstart, dtend, href, ref, dtype, rec_inst, calendar)'
                     'VALUES (?, ?, ?, ?, ?, ?, ?);'.format(recs_table))
-                stuple = (dbstart, dbend, href, ref, dtype, rec_inst, calendar)
-            self.sql_ex(recs_sql_s, stuple)
+                stuple_n = (dbstart, dbend, href, ref, dtype, rec_inst, calendar)
+                self.sql_ex(recs_sql_s, stuple_n)
 
-    def get_ctag(self, calendar):
+    def get_ctag(self, calendar=str) -> Optional[str]:
         stuple = (calendar, )
         sql_s = 'SELECT ctag FROM calendars WHERE calendar = ?;'
         try:
@@ -395,13 +400,13 @@ class SQLiteDb(object):
         except IndexError:
             return None
 
-    def set_ctag(self, ctag, calendar):
+    def set_ctag(self, ctag: str, calendar: str):
         stuple = (ctag, calendar, )
         sql_s = 'UPDATE calendars SET ctag = ? WHERE calendar = ?;'
         self.sql_ex(sql_s, stuple)
         self.conn.commit()
 
-    def get_etag(self, href, calendar):
+    def get_etag(self, href: str, calendar: str) -> Optional[str]:
         """get etag for href
 
         type href: str()
@@ -415,7 +420,7 @@ class SQLiteDb(object):
         except IndexError:
             return None
 
-    def delete(self, href, etag=None, calendar=None):
+    def delete(self, href: str, etag: Any=None, calendar: str=None):
         """
         removes the event from the db,
 
@@ -430,6 +435,23 @@ class SQLiteDb(object):
         sql_s = 'DELETE FROM events WHERE href = ? AND calendar = ?;'
         self.sql_ex(sql_s, (href, calendar))
 
+    def deletelike(self, href: str, etag: Any=None, calendar: str=None):
+        """
+        removes events from the db that match an SQL 'like' statement,
+
+        :param href: The pattern of hrefs to delete. May contain SQL wildcards
+                     like '%'
+        :param etag: only there for compatibility with vdirsyncer's Storage,
+                     we always delete
+        :returns: None
+        """
+        assert calendar is not None
+        for table in ['recs_loc', 'recs_float']:
+            sql_s = 'DELETE FROM {0} WHERE href LIKE ? AND calendar = ?;'.format(table)
+            self.sql_ex(sql_s, (href, calendar))
+        sql_s = 'DELETE FROM events WHERE href LIKE ? AND calendar = ?;'
+        self.sql_ex(sql_s, (href, calendar))
+
     def list(self, calendar):
         """ list all events in `calendar`
 
@@ -439,7 +461,28 @@ class SQLiteDb(object):
         sql_s = 'SELECT href, etag FROM events WHERE calendar = ?;'
         return list(set(self.sql_ex(sql_s, (calendar, ))))
 
-    def get_localized(self, start, end, minimal=False):
+    def get_localized_calendars(self, start: dt.datetime, end: dt.datetime) -> Iterable[str]:
+        assert start.tzinfo is not None
+        assert end.tzinfo is not None
+        start_u = utils.to_unix_time(start)
+        end_u = utils.to_unix_time(end)
+        sql_s = (
+            'SELECT events.calendar FROM '
+            'recs_loc JOIN events ON '
+            'recs_loc.href = events.href AND '
+            'recs_loc.calendar = events.calendar WHERE '
+            '(dtstart >= ? AND dtstart <= ? OR '
+            'dtend > ? AND dtend <= ? OR '
+            'dtstart <= ? AND dtend >= ?) AND events.calendar in ({0}) '
+            'ORDER BY dtstart')
+        stuple = tuple(
+            [start_u, end_u, start_u, end_u, start_u, end_u] + list(self.calendars))  # type: ignore
+        result = self.sql_ex(sql_s.format(','.join(["?"] * len(self.calendars))), stuple)
+        for calendar in result:
+            yield calendar[0]  # result is always an iterable, even if getting only one item
+
+    def get_localized(self, start, end) \
+            -> Iterable[Tuple[str, str, dt.datetime, dt.datetime, str, str, str]]:
         """returns
         :type start: datetime.datetime
         :type end: datetime.datetime
@@ -450,118 +493,82 @@ class SQLiteDb(object):
         assert end.tzinfo is not None
         start = utils.to_unix_time(start)
         end = utils.to_unix_time(end)
-        if minimal:
-            sql_s = (
-                'SELECT events.calendar FROM '
-                'recs_loc JOIN events ON '
-                'recs_loc.href = events.href AND '
-                'recs_loc.calendar = events.calendar WHERE '
-                '(dtstart >= ? AND dtstart <= ? OR '
-                'dtend > ? AND dtend <= ? OR '
-                'dtstart <= ? AND dtend >= ?) AND events.calendar in ({0}) '
-                'ORDER BY dtstart')
-        else:
-            sql_s = (
-                'SELECT item, recs_loc.href, dtstart, dtend, ref, etag, dtype, events.calendar '
-                'FROM recs_loc JOIN events ON '
-                'recs_loc.href = events.href AND '
-                'recs_loc.calendar = events.calendar WHERE '
-                '(dtstart >= ? AND dtstart <= ? OR '
-                'dtend > ? AND dtend <= ? OR '
-                'dtstart <= ? AND dtend >= ?) AND events.calendar in ({0}) '
-                'ORDER BY dtstart')
-        stuple = (start, end, start, end, start, end)
-        result = self.sql_ex(sql_s.format(self._select_calendars), stuple)
-        if minimal:
-            for calendar in result:
-                yield EventStandIn(calendar[0])
-        else:
-            for item, href, start, end, ref, etag, dtype, calendar in result:
-                start = pytz.UTC.localize(datetime.utcfromtimestamp(start))
-                end = pytz.UTC.localize(datetime.utcfromtimestamp(end))
-                yield self.construct_event(item, href, start, end, ref, etag, calendar, dtype)
+        sql_s = (
+            'SELECT item, recs_loc.href, dtstart, dtend, ref, etag, dtype, events.calendar '
+            'FROM recs_loc JOIN events ON '
+            'recs_loc.href = events.href AND '
+            'recs_loc.calendar = events.calendar WHERE '
+            '(dtstart >= ? AND dtstart <= ? OR '
+            'dtend > ? AND dtend <= ? OR '
+            'dtstart <= ? AND dtend >= ?) AND events.calendar in ({0}) '
+            'ORDER BY dtstart')
+        stuple = tuple([start, end, start, end, start, end] + list(self.calendars))
+        result = self.sql_ex(sql_s.format(','.join(["?"] * len(self.calendars))), stuple)
+        for item, href, start, end, ref, etag, dtype, calendar in result:
+            start = pytz.UTC.localize(dt.datetime.utcfromtimestamp(start))
+            end = pytz.UTC.localize(dt.datetime.utcfromtimestamp(end))
+            yield item, href, start, end, ref, etag, calendar
 
-    def get_floating(self, start, end, minimal=False):
+    def get_floating_calendars(self, start: dt.datetime, end: dt.datetime) -> Iterable[str]:
+        assert start.tzinfo is None
+        assert end.tzinfo is None
+        start_u = utils.to_unix_time(start)
+        end_u = utils.to_unix_time(end)
+        sql_s = (
+            'SELECT events.calendar FROM '
+            'recs_float JOIN events ON '
+            'recs_float.href = events.href AND '
+            'recs_float.calendar = events.calendar WHERE '
+            '(dtstart >= ? AND dtstart < ? OR '
+            'dtend > ? AND dtend <= ? OR '
+            'dtstart <= ? AND dtend > ? ) AND events.calendar in ({0}) '
+            'ORDER BY dtstart')
+        stuple = tuple(
+            [start_u, end_u, start_u, end_u, start_u, end_u] + list(self.calendars))  # type: ignore
+        result = self.sql_ex(sql_s.format(','.join(["?"] * len(self.calendars))), stuple)
+        for calendar in result:
+            yield calendar[0]
+
+    def get_floating(self, start, end) \
+            -> Iterable[Tuple[str, str, dt.datetime, dt.datetime, str, str, str]]:
         """return floating events between `start` and `end`
 
         :type start: datetime.datetime
         :type end: datetime.datetime
-        :param minimal: if set, we do not return an event but a minimal stand in
-        :type minimal: bool
         """
         assert start.tzinfo is None
         assert end.tzinfo is None
-        strstart = utils.to_unix_time(start)
-        strend = utils.to_unix_time(end)
-        if minimal:
-            sql_s = (
-                'SELECT events.calendar FROM '
-                'recs_float JOIN events ON '
-                'recs_float.href = events.href AND '
-                'recs_float.calendar = events.calendar WHERE '
-                '(dtstart >= ? AND dtstart < ? OR '
-                'dtend > ? AND dtend <= ? OR '
-                'dtstart <= ? AND dtend > ? ) AND events.calendar in ({0}) '
-                'ORDER BY dtstart')
-        else:
-            sql_s = (
-                'SELECT item, recs_float.href, dtstart, dtend, ref, etag, dtype, events.calendar '
-                'FROM recs_float JOIN events ON '
-                'recs_float.href = events.href AND '
-                'recs_float.calendar = events.calendar WHERE '
-                '(dtstart >= ? AND dtstart < ? OR '
-                'dtend > ? AND dtend <= ? OR '
-                'dtstart <= ? AND dtend > ? ) AND events.calendar in ({0}) '
-                'ORDER BY dtstart')
-        stuple = (strstart, strend, strstart, strend, strstart, strend)
-        result = self.sql_ex(sql_s.format(self._select_calendars), stuple)
-        if minimal:
-            for calendar in result:
-                yield EventStandIn(calendar[0])
-        else:
-            for item, href, start, end, ref, etag, dtype, calendar in result:
-                start = datetime.utcfromtimestamp(start)
-                end = datetime.utcfromtimestamp(end)
-                yield self.construct_event(item, href, start, end, ref, etag, calendar, dtype)
+        start_u = utils.to_unix_time(start)
+        end_u = utils.to_unix_time(end)
+        sql_s = (
+            'SELECT item, recs_float.href, dtstart, dtend, ref, etag, dtype, events.calendar '
+            'FROM recs_float JOIN events ON '
+            'recs_float.href = events.href AND '
+            'recs_float.calendar = events.calendar WHERE '
+            '(dtstart >= ? AND dtstart < ? OR '
+            'dtend > ? AND dtend <= ? OR '
+            'dtstart <= ? AND dtend > ? ) AND events.calendar in ({0}) '
+            'ORDER BY dtstart')
+        stuple = tuple(
+            [start_u, end_u, start_u, end_u, start_u, end_u] + list(self.calendars))  # type: ignore
+        result = self.sql_ex(sql_s.format(','.join(["?"] * len(self.calendars))), stuple)
+        for item, href, start, end, ref, etag, dtype, calendar in result:
+            start = dt.datetime.utcfromtimestamp(start)
+            end = dt.datetime.utcfromtimestamp(end)
+            if dtype == EventType.DATE:
+                start = start.date()
+                end = end.date()
+            yield item, href, start, end, ref, etag, calendar
 
-    def get(self, href, start=None, end=None, ref=None, dtype=None, calendar=None):
-        """returns the Event matching href
-
-        if start and end are given, a specific Event from a Recursion set is
-        returned, otherwise the Event returned exactly as saved in the db
-        """
+    def get(self, href: str, calendar: str) -> str:
+        """returns the ical string matching href and calendar"""
         assert calendar is not None
-        sql_s = 'SELECT href, etag, item FROM events WHERE href = ? AND calendar = ?;'
-        result = self.sql_ex(sql_s, (href, calendar))
-        href, etag, item = result[0]
-        if dtype == DATE:
-            start = start.date()
-            end = end.date()
-        return Event.fromString(item,
-                                locale=self.locale,
-                                href=href,
-                                calendar=calendar,
-                                etag=etag,
-                                start=start,
-                                end=end,
-                                ref=ref,
-                                )
+        sql_s = 'SELECT item, etag FROM events WHERE href = ? AND calendar = ?;'
+        item, etag = self.sql_ex(sql_s, (href, calendar))[0]
+        return item
 
-    def construct_event(self, item, href, start, end, ref, etag, calendar, dtype=None):
-        if dtype == DATE:
-            start = start.date()
-            end = end.date()
-        return Event.fromString(item,
-                                locale=self.locale,
-                                href=href,
-                                calendar=calendar,
-                                etag=etag,
-                                start=start,
-                                end=end,
-                                ref=ref,
-                                )
-
-    def search(self, search_string):
+    def search(self, search_string: str) \
+            -> Iterable[Tuple[str, str, dt.datetime, dt.datetime, str, str, str]]:
         """search for events matching `search_string`"""
         sql_s = (
             'SELECT item, recs_loc.href, dtstart, dtend, ref, etag, dtype, events.calendar '
@@ -570,12 +577,15 @@ class SQLiteDb(object):
             'recs_loc.calendar = events.calendar '
             'WHERE item LIKE (?) and events.calendar in ({0});'
         )
-        stuple = ('%{0}%'.format(search_string), )
-        result = self.sql_ex(sql_s.format(self._select_calendars), stuple)
+        stuple = tuple(['%{0}%'.format(search_string)] + list(self.calendars))
+        result = self.sql_ex(sql_s.format(','.join(["?"] * len(self.calendars))), stuple)
         for item, href, start, end, ref, etag, dtype, calendar in result:
-            start = pytz.UTC.localize(datetime.utcfromtimestamp(start))
-            end = pytz.UTC.localize(datetime.utcfromtimestamp(end))
-            yield self.construct_event(item, href, start, end, ref, etag, calendar, dtype)
+            start = pytz.UTC.localize(dt.datetime.utcfromtimestamp(start))
+            end = pytz.UTC.localize(dt.datetime.utcfromtimestamp(end))
+            if dtype == EventType.DATE:
+                start = start.date()
+                end = end.date()
+            yield item, href, start, end, ref, etag, calendar
 
         sql_s = (
             'SELECT item, recs_float.href, dtstart, dtend, ref, etag, dtype, events.calendar '
@@ -584,21 +594,22 @@ class SQLiteDb(object):
             'recs_float.calendar = events.calendar '
             'WHERE item LIKE (?) and events.calendar in ({0});'
         )
-        stuple = ('%{0}%'.format(search_string), )
-        result = self.sql_ex(sql_s.format(self._select_calendars), stuple)
+        stuple = tuple(['%{0}%'.format(search_string)] + list(self.calendars))
+        result = self.sql_ex(sql_s.format(','.join(["?"] * len(self.calendars))), stuple)
         for item, href, start, end, ref, etag, dtype, calendar in result:
-            start = datetime.utcfromtimestamp(start)
-            end = datetime.utcfromtimestamp(end)
-            yield self.construct_event(item, href, start, end, ref, etag, calendar, dtype)
+            start = dt.datetime.utcfromtimestamp(start)
+            end = dt.datetime.utcfromtimestamp(end)
+            if dtype == EventType.DATE:
+                start = start.date()
+                end = end.date()
+            yield item, href, start, end, ref, etag, calendar
 
 
-def check_support(vevent, href, calendar):
+def check_support(vevent: icalendar.cal.Event, href: str, calendar: str):
     """test if all icalendar features used in this event are supported,
     raise `UpdateFailed` otherwise.
     :param vevent: event to test
-    :type vevent: icalendar.cal.Event
     :param href: href of this event, only used for logging
-    :type href: str
     """
     rec_id = vevent.get(RECURRENCE_ID)
 
@@ -621,7 +632,7 @@ def check_support(vevent, href, calendar):
         )
 
 
-def check_for_errors(component, calendar, href):
+def check_for_errors(component: icalendar.cal.Component, calendar: str, href: str):
     """checking if component.errors exists, is not empty and if so warn the user"""
     if hasattr(component, 'errors') and component.errors:
         logger.error(
@@ -632,14 +643,11 @@ def check_for_errors(component, calendar, href):
         logger.error('This might lead to this event being shown wrongly or not at all.')
 
 
-def calc_shift_deltas(vevent):
+def calc_shift_deltas(vevent: icalendar.Event) -> Tuple[dt.timedelta, dt.timedelta]:
     """calculate an event's duration and by how much its start time has shifted
     versus its recurrence-id time
 
     :param event: an event with an RECURRENCE-ID property
-    :type event: icalendar.Event
-    :returns: time shift and duration
-    :rtype: (datetime.timedelta, datetime.timedelta)
     """
     assert isinstance(vevent, icalendar.Event)  # REMOVE ME
     start_shift = vevent['DTSTART'].dt - vevent['RECURRENCE-ID'].dt
@@ -648,3 +656,22 @@ def calc_shift_deltas(vevent):
     except KeyError:
         duration = vevent['DURATION'].dt
     return start_shift, duration
+
+
+def get_vcard_event_description(vcard: icalendar.cal.Component, key: str) -> str:
+    if key == 'BDAY':
+        return 'birthday'
+    elif key.endswith('ANNIVERSARY'):
+        return 'anniversary'
+    elif key.endswith('X-ABDATE'):
+        desc_key = key[:-8] + 'X-ABLABEL'
+        if desc_key in vcard.keys():
+            return vcard[desc_key]
+        else:
+            desc_key = key[:-8] + 'X-ABLabel'
+            if desc_key in vcard.keys():
+                return vcard[desc_key]
+            else:
+                return 'custom event from vcard'
+    else:
+        return 'unknown event from vcard'
